----------------------------------------------------------------------------------------------------
-- DotFocus
-- Author: Snowcloud (Micâh - Antonidas)
-- Codebase by: Meranannon (Insane - Vol'jin)
----------------------------------------------------------------------------------------------------

local DF = DOF_namespace
local _

local state={}
local letype={}
local scale={}

local frame=nil
local frameTexture=nil
local overlay={}
local overlaybase={}
local overlayTexture={}

local frametext = {}
local frametext_overlay = {}	
local frametext_text = {}	

local frequency =1/60
local tempo=(frequency/15)*3

local SchoolStringTable = {
    -- Single Schools
    [SCHOOL_MASK_PHYSICAL]                      = STRING_SCHOOL_PHYSICAL,
    [SCHOOL_MASK_HOLY]                          = STRING_SCHOOL_HOLY,
    [SCHOOL_MASK_FIRE]                          = STRING_SCHOOL_FIRE,
    [SCHOOL_MASK_NATURE]                        = STRING_SCHOOL_NATURE,
    [SCHOOL_MASK_FROST]                         = STRING_SCHOOL_FROST,
    [SCHOOL_MASK_SHADOW]                        = STRING_SCHOOL_SHADOW,
    [SCHOOL_MASK_ARCANE]                        = STRING_SCHOOL_ARCANE,
    -- Physical and a Magical
    [SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_FIRE]   = STRING_SCHOOL_FLAMESTRIKE,
    [SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_FROST]  = STRING_SCHOOL_FROSTSTRIKE,
    [SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_ARCANE] = STRING_SCHOOL_SPELLSTRIKE,
    [SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_NATURE] = STRING_SCHOOL_STORMSTRIKE,
    [SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_SHADOW] = STRING_SCHOOL_SHADOWSTRIKE,
    [SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_HOLY]   = STRING_SCHOOL_HOLYSTRIKE,
    -- Two Magical Schools
    [SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST]      = STRING_SCHOOL_FROSTFIRE,
    [SCHOOL_MASK_FIRE + SCHOOL_MASK_ARCANE]     = STRING_SCHOOL_SPELLFIRE,
    [SCHOOL_MASK_FIRE + SCHOOL_MASK_NATURE]     = STRING_SCHOOL_FIRESTORM,
    [SCHOOL_MASK_FIRE + SCHOOL_MASK_SHADOW]     = STRING_SCHOOL_SHADOWFLAME,
    [SCHOOL_MASK_FIRE + SCHOOL_MASK_HOLY]       = STRING_SCHOOL_HOLYFIRE,
    [SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE]    = STRING_SCHOOL_SPELLFROST,
    [SCHOOL_MASK_FROST + SCHOOL_MASK_NATURE]    = STRING_SCHOOL_FROSTSTORM,
    [SCHOOL_MASK_FROST + SCHOOL_MASK_SHADOW]    = STRING_SCHOOL_SHADOWFROST,
    [SCHOOL_MASK_FROST + SCHOOL_MASK_HOLY]      = STRING_SCHOOL_HOLYFROST,
    [SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE]   = STRING_SCHOOL_SPELLSTORM,
    [SCHOOL_MASK_ARCANE + SCHOOL_MASK_SHADOW]   = STRING_SCHOOL_SPELLSHADOW,
    [SCHOOL_MASK_ARCANE + SCHOOL_MASK_HOLY]     = STRING_SCHOOL_DIVINE,
    [SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW]   = STRING_SCHOOL_SHADOWSTORM,
    [SCHOOL_MASK_NATURE + SCHOOL_MASK_HOLY]     = STRING_SCHOOL_HOLYSTORM,
    [SCHOOL_MASK_SHADOW + SCHOOL_MASK_HOLY]     = STRING_SCHOOL_SHADOWLIGHT,
    -- Three or more schools
    [SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_NATURE]                                                                                     = STRING_SCHOOL_ELEMENTAL,
    [SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW]                                           = STRING_SCHOOL_CHROMATIC,
    [SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW + SCHOOL_MASK_HOLY]                        = STRING_SCHOOL_MAGIC,
    [SCHOOL_MASK_PHYSICAL + SCHOOL_MASK_FIRE + SCHOOL_MASK_FROST + SCHOOL_MASK_ARCANE + SCHOOL_MASK_NATURE + SCHOOL_MASK_SHADOW + SCHOOL_MASK_HOLY] = STRING_SCHOOL_CHAOS,
};
 
local function DFalert_SchoolString(school)
    if ( not school or school == SCHOOL_MASK_NONE ) then
        return STRING_SCHOOL_UNKNOWN;
    end
 
    local schoolString = SchoolStringTable[school];
    return schoolString or STRING_SCHOOL_UNKNOWN
end

-- initialisation frames / initilization frames
function DF:init_alert_frame()
	
	local anchorx
	local anchory
	local i
	
	if not frame then
		
		-- cadre principal / main frame
		frame = CreateFrame("FRAME","DF_ALERT_FRAME",UIParent)

		frame:SetScript("OnMouseDown",function(self,button)
			if button=="LeftButton" then
  			frame:StartMoving()
  		elseif button=="RightButton" then
  			DF:options_show("DFalert",frame)
  		end
  	end)
		frame:SetScript("OnMouseUp",function(self,button)
			if button=="LeftButton" then
	  		frame:StopMovingOrSizing()

	  		local anchorx=DF.anchor[1].frame:GetLeft()
	  		local anchory=DF.anchor[1].frame:GetTop()
	  		DOF_config.alert.positionx=self:GetLeft()-anchorx
	  		DOF_config.alert.positiony=self:GetTop()-anchory

	  		frame:ClearAllPoints()
	  		frame:SetPoint("TOPLEFT", DF.anchor[1].frame, "TOPLEFT", DOF_config.alert.positionx, DOF_config.alert.positiony)
				DF:updateWidgetValue("DFalertleft")
				DF:updateWidgetValue("DFalerttop")

		  end
		end)	
		frame:SetScript("OnEnter",function(self,button)
			if DF.configmode then
				DF.DotFGameTooltip:SetOwner(UIParent, "ANCHOR_TOPLEFT ",16,-16)
				DF.DotFGameTooltip:ClearLines()
				DF.DotFGameTooltip:AddLine("DOTFOCUS ALERT",1,1,0,nil)
				DF.DotFGameTooltip:AddLine(DF.locale["leftMB"],1,1,1,nil)
				DF.DotFGameTooltip:AddLine(DF.locale["rightMB"],1,1,1,nil)
				DF.DotFGameTooltip:Show()		
			end		
		end)		
		frame:SetScript("OnLeave",function(self,button)
			if DF.configmode then DF.DotFGameTooltip:Hide() end
		end)		

		frameTexture = frame:CreateTexture("DF_ALERT_FRAME_TEXTURE","BACKGROUND")

		for i = 1,4 do	
			state[i]=0
			scale[i]=1
			
			overlay[i]={}	
			overlaybase[i]={}	
			frametext_overlay[i]={}
			frametext_text[i]={}
			
			-- cadre pour la texture / framework for the texture
			overlaybase[i] = CreateFrame("FRAME","DF_ALERT_OVERLAYBASE"..tostring(i),frame)
			overlay[i] = CreateFrame("FRAME","DF_ALERT_OVERLAY"..tostring(i),overlaybase[i])
			
			-- la texture / texture
			overlayTexture[i] = overlay[i]:CreateTexture("DF_ALERT_OVERLAY_TEXTURE"..tostring(i),"BACKGROUND")
	
			-- le text / text
			frametext_overlay[i] = CreateFrame("FRAME","DF_ALERT_FRAMETEXT_OVERLAY"..tostring(i),overlaybase[i])
			frametext_text[i] = frametext_overlay[i]:CreateFontString("DF_ALERT_FRAMETEXT_TEXT"..tostring(i),"ARTWORK")		
		end
		
		frame:EnableMouse(false)	
			
	end
	
	if not DOF_config.alert.newLevel then
		DOF_config.alert.newLevel=DOF_config.alert.level*10
	end
	local level = DOF_config.alert.newLevel

	-- paramétres cadre principal / main frame parameters
	frame:SetMovable(true)
	
	frame:SetWidth(64)
	frame:SetHeight(64)
	frame:ClearAllPoints()
	frame:SetPoint("TOPLEFT", DF.anchor[1].frame, "TOPLEFT", DOF_config.alert.positionx, DOF_config.alert.positiony)
	--frame:SetFrameLevel(level)
	--frame:SetFrameLevel(level)
	
	-- paramétres texture / texture parameters
	frameTexture:SetTexCoord(0, 1, 0, 1)
	frameTexture:ClearAllPoints()
	frameTexture:SetAllPoints(frame)
	
	frameTexture:SetColorTexture(1,1,1,0)

	for i = 1,4 do		

		overlaybase[i]:SetMovable(false)
		overlaybase[i]:EnableMouse(false)		
		overlaybase[i]:SetWidth(DOF_config.alert.width)
		overlaybase[i]:SetHeight(DOF_config.alert.height)
		overlaybase[i]:SetPoint("CENTER", frame, "CENTER", (DOF_config.alert.decalx*(i-1)), (DOF_config.alert.decaly*(i-1)))
		--overlaybase[i]:SetFrameLevel(level+1)

		overlay[i]:SetMovable(false)
		overlay[i]:EnableMouse(false)		
		overlay[i]:SetWidth(DOF_config.alert.width)
		overlay[i]:SetHeight(DOF_config.alert.height)
		overlay[i]:SetPoint("CENTER", overlaybase[i], "CENTER", 0,0)
		--overlay[i]:SetFrameLevel(level+2)
		
		overlay[i]:SetAlpha(DOF_config.alert.alpha)
		
		-- paramétres texture / texture parameters
		overlayTexture[i]:SetTexCoord(0, 1, 0, 1)
		overlayTexture[i]:SetWidth(128)
		overlayTexture[i]:SetHeight(128)
		overlayTexture[i]:SetBlendMode(DOF_config.alert.mode)
		overlayTexture[i]:ClearAllPoints()
		overlayTexture[i]:SetAllPoints(overlay[i])
		overlayTexture[i]:SetTexture(DOF_config.alert.texture1)
		
		-- place la texture dans le cadre / place the texture in
		overlay.texture = overlayTexture[i]
	
		-- parametre texte / text parameter
		frametext_overlay[i]:SetWidth(256)
		frametext_overlay[i]:SetHeight(32)
		frametext_overlay[i]:ClearAllPoints()
		frametext_overlay[i]:SetPoint("CENTER", overlay[i], "CENTER", 0, 0)
		--frametext_overlay[i]:SetFrameLevel(level+2)	
		
		DF:MySetFont(frametext_text[i],DOF_config.alert.fontPath,DOF_config.alert.fontSize,"OUTLINE")
		frametext_text[i]:SetWidth(256)
		frametext_text[i]:SetHeight(32)	
		frametext_text[i]:SetJustifyH('CENTER')
		frametext_text[i]:SetJustifyV('MIDDLE')	
		frametext_text[i]:SetShadowColor(0, 0, 0, 0.75)
		frametext_text[i]:SetShadowOffset(0.5, -0.5)
		frametext_text[i]:SetTextColor(DOF_config.alert.textColor.r, DOF_config.alert.textColor.v, DOF_config.alert.textColor.b, DOF_config.alert.textColor.a)
		frametext_text[i]:ClearAllPoints()
		frametext_text[i]:SetPoint("CENTER", frametext_overlay[i],"CENTER", DOF_config.alert.textOffsetX,DOF_config.alert.textOffsetY)
		frametext_text[i]:SetText("")
		
		overlay[i]:Hide()
	
	end	
	
end

-- gestion de l'animation / animation management
function DF:alert_update(elapsed)

	local tempScale

	if not DOF_config.alert.enable then
		frame:Hide()
		return
	else
		frame:Show()	
	end

	tempo=tempo+elapsed
	if tempo<frequency then return end
	tempo=0		

	if DF.configmode then
		frameTexture:SetColorTexture(1,1,1,0.25)
	else
		frameTexture:SetColorTexture(1,1,1,0)
	end

	if DF.configmode then
		
		for i = 1,4 do	
			overlayTexture[i]:SetTexture("Interface\\AddOns\\DotFocus\\datas\\alertBehind.tga")
			overlay[i]:SetScale(1)
			if DOF_config.alert.showText then
				frametext_text[i]:SetText("Spell Name\n(Spell School)\n(BUFF/DEBUFF)")
				frametext_text[i]:Show()
				frametext_overlay[i]:Show()
			end
			overlay[i]:Show()
		end
		
	else
		
		for i = 1,4 do	
	
			if state[i] == 1 then
			
				if scale[i]>1 then
					scale[i]  = scale[i]  - elapsed
				else
					scale[i]  = scale[i]  - 0.05
				end
			
				tempScale=scale[i]
			
				if	tempScale>1 then tempScale=1 end			
				if	tempScale<0.1 then
					tempScale=0.1
					state[i]=0
					letype[i]="0"
				end
							
				overlay[i]:SetScale(DF:smoothScale(tempScale));
				overlay[i]:SetAlpha(DF:smoothAlpha(tempScale,DOF_config.alert.alpha));
				overlay[i]:Show();		

				if tempScale>0.75 then
					frametext_overlay[i]:Show()
				else
					frametext_overlay[i]:Hide()
				end
				
			elseif state[i]==2 then
				
				scale[i]  = scale[i]  + 0.05
		
				if	scale[i]>1 then
					
					scale[i]=DOF_config.alert.persistence
					state[i]=1
					
				end

				tempScale=scale[i]
				if	tempScale>1 then tempScale=1 end		
				if	tempScale<0.01 then tempScale=0.01 end
				
				overlay[i]:SetScale(DF:smoothScale(tempScale));
				overlay[i]:SetAlpha(DF:smoothAlpha(tempScale,DOF_config.alert.alpha));
				overlay[i]:Show();	

				if tempScale>0.75 then
					frametext_overlay[i]:Show()
				else
					frametext_overlay[i]:Hide()
				end
					
			else
				
				overlay[i]:Hide()
				frametext_text[i]:SetText("")
				
			end	
	
		end
	end

end

function DF:alert_activate(aType,force,spellname,spellschool,spelltype)

	local spellcolor
	local num=0
	
	if not DOF_config.alert.enable then
		return
	end	

	local alertExist=false
	for i = 1,4 do	
		if state[i]~=0 then
			if letype[i]==aType then
				alertExist=true
			end
		end
	end
	
	if not alertExist or force then
		
		for i = 1,4 do	
			if state[i]==0 then
				num=i
				break
			end
		end
	
		if num>0 then
		
			if aType=="1" then
				overlayTexture[num]:SetTexture(DOF_config.alert.texture1)
				frametext_text[num]:SetText("")
				letype[num]="1"
			elseif aType=="2" then
				overlayTexture[num]:SetTexture(DOF_config.alert.texture2)
				frametext_text[num]:SetText("")
				letype[num]="2"
			elseif aType=="3" then
				overlayTexture[num]:SetTexture(DOF_config.alert.texture3)
				frametext_text[num]:SetText("")
				letype[num]="3"
			else
				overlayTexture[num]:SetTexture(aType)
				letype[num]=aType
				if spellname and DOF_config.alert.showText then
					spellcolor="|cFF0088FF"
					if spelltype=="BUFF" then
						spellcolor="|cFF00FF88"
					elseif spelltype=="DEBUFF" then
						spellcolor="|cFFFF8800"
					end
					if spellschool~="" then
						frametext_text[num]:SetText(spellcolor..spellname.."|n"..DFalert_SchoolString(spellschool).."|n|cFFFFFFFF"..spelltype)
					else
						frametext_text[num]:SetText(spellcolor..spellname.."|n|cFFFFFFFF"..spelltype)
					end
				end
			end
		
			state[num]=2
			scale[num]=0.1
			
		end
		
	end
	
end

-- enable/disable déplacement du cadre avec la souris
-- enable/disable frame moving with the mouse
function DF:alert_toogle_lock(flag)
	
	frame:EnableMouse(flag)
	
end

function DF:alert_reinit()
	
	DF:init_alert_frame()
	DF:alert_activate("1",true)
	DF:alert_toogle_lock(DF.configmode)
	DF:options_setLevel()
	
end

function DF:alert_setLevel()
	DF:setMyFrameLevel(frame,DOF_config.alert.newLevel,"alert")
	for i = 1,4 do
		DF:setMyFrameLevel(overlaybase[i],DOF_config.alert.newLevel+1)
		DF:setMyFrameLevel(overlay[i],DOF_config.alert.newLevel+2)
		DF:setMyFrameLevel(frametext_overlay[i],DOF_config.alert.newLevel+2)
	end
end